{ lib
, nixpkgsOnBuildForBuild
, nixpkgsOnBuildForHost
, coreboot-toolchain       ? throw "missing"
, coreboot ? throw "missing"
, payload                  ? throw "you must provide a coreboot payload (Linux kernel image or FIT)"
, payload-compression-type ? "none"
, payload-prefix           ? "prefix"
, payload-name             ? "payload"
, initramfs                ? null # initramfs `cpio` archive
, linux-command-line       ? null
, images-per-flash-chip
}:

nixpkgsOnBuildForHost.stdenv.mkDerivation {
  pname = "ownerboot";
  inherit (coreboot) version;

  passthru = { inherit (coreboot) src fmap; };

  nativeBuildInputs = [
    nixpkgsOnBuildForBuild.coreboot-utils
  ];

  dontUnpack = true;

  buildPhase = let
    update-cbfs = region: lib.concatStringsSep " " ([
      "cbfstool"
      "coreboot.rom"
      "add-payload"
      "-r${region}"
      "-f${payload}"
      "-n${payload-prefix}/${payload-name}"
      "-c${payload-compression-type}"
    ] ++ lib.optionals (linux-command-line != null) [
      "-C'${linux-command-line}'"
    ] ++ lib.optionals (initramfs != null) [
      "-I'${initramfs}'"
    ] ++ [
      "\n"
    ]);
    in (''
      runHook preBuild
      cp ${coreboot}/coreboot.rom .
      chmod +w coreboot.rom

      ${update-cbfs "FALLBACK"}
      cbfstool coreboot.rom print -rFALLBACK

    '' + lib.optionalString (images-per-flash-chip > 1) ''
      # unfortunately this is the only way to trick cbfstool into
      # putting a "header pointer" in both regions:
      cbfstool coreboot.rom read     -r FALLBACK -f half.img
      cbfstool coreboot.rom write -F -r NORMAL   -f half.img
      rm half.img
      cbfstool coreboot.rom print -rNORMAL
    '' + ''

      runHook postBuild
    '');

  installPhase = ''
    runHook preBuild
    mkdir -p $out
    cp -a ${coreboot}/* $out/
    chmod +w $out/coreboot.rom
    rm $out/coreboot.rom
    mv coreboot.rom $out/
    runHook postBuild
  '';

  dontFixup = true;

}
